from .impl.datasets.loaders import get_data_home
from .impl.datasets.loaders import load_news20
from .impl.datasets.loaders import load_usps
from .impl.datasets.loaders import load_mnist
from .impl.datasets.loaders import get_loader
from .impl.datasets.loaders import load_dataset
from .impl.datasets.utils import make_ovo
from .impl.datasets.samples_generator import make_nn_regression
from .impl.datasets.samples_generator import make_classification
