#  tests for sklearn-contrib-lightning-0.6.0-py36he38d939_0 (this is a generated file);
print('===== testing package: sklearn-contrib-lightning-0.6.0-py36he38d939_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import lightning
import nose
import os

lightning_dir = os.path.dirname(
    os.path.abspath(lightning.__file__))
os.chdir(lightning_dir)
nose.run(module=lightning)
#  --- run_test.py (end) ---

print('===== sklearn-contrib-lightning-0.6.0-py36he38d939_0 OK =====');
print("import: 'lightning'")
import lightning

